#ifndef KADDRESSBOOKTABLEVIEW_H
#define KADDRESSBOOKTABLEVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <qwidget.h>
#include <qlistview.h>
#include <qstring.h>
#include <qdialog.h>
#include <qpixmap.h>
#include <qtabdialog.h>
#include <qfileinfo.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qtooltip.h>

#include "undo.h"
#include "kaddressbookview.h"

class QListViewItem;
class QListBox;
class QVBoxLayout;
class KConfig;

class ContactEntry;
class ContactEntryList;
class ContactListViewItem;
class ContactListView;

/**
 * This class is the table view for kaddressbook. This view is a KListView
 * with multiple columns for the selected fields.
 *
 * @short Table View
 * @author Don Sanders <dsanders@kde.org>
 * @version 0.1
 */
class KAddressBookTableView : public KAddressBookView
{
friend class ContactListView;

  Q_OBJECT

public:
  KAddressBookTableView(ContactEntryList *cel, QWidget *parent, 
                        const char *name = 0L );
  virtual ~KAddressBookTableView();
  
  /** Returns the contactEntryList used in this class to store a list
  * of the contact entries.
  */
  virtual ContactEntryList* contactEntryList();
  
  /** Returns the list view used in this class.
  */
  virtual ContactListView* listView();
  
  /** NEW CODE **/
  virtual void refresh(QString uid = QString::null);
  virtual QStringList selectedUids();
  virtual void setSelected(QString uid = QString::null, bool selected = false);
  virtual void readConfig(KConfig *config);
  virtual void writeConfig(KConfig *config);
  virtual QString type() const { return "Table"; }
  /** END NEW CODE **/

public slots:
  //virtual void showSelectNameDialog();
  //virtual void defaultSettings(); 
  virtual void reconstructListView();
  void change( QString entryKey, ContactEntry *ce );

protected slots:
  /** Called whenever the user selects an addressee in the list view.
  */
  void addresseeSelected();
  
  /** Called whenever the user executes an addressee. In terms of the
  * list view, this is probably a double click
  */
  void addresseeExecuted(QListViewItem*);
  
  //void viewOptions();
 
protected:
  virtual void incrementalSearch(const QString &value,KABC::Field *field);
  
  //virtual void selectNames( QStringList fields );
 
  ContactEntryList *mDocument;
  QVBoxLayout *mainLayout;
  ContactListView *mListView;
};

#endif
