#ifndef UNDOCMDS_H
#define UNDOCMDS_H
// $Id: undocmds.h,v 1.3 2002/04/15 00:10:03 mpilone Exp $
//
// Commands for undo/redo functionality.

#include <qstring.h>
#include <qstringlist.h>

#include "undo.h"

class ContactEntry;
class ContactEntryList;

class PwDeleteCommand : public Command
{
public:
  PwDeleteCommand(ContactEntryList *doc, QString entryKey, 
                  ContactEntry *ce );
  virtual ~PwDeleteCommand();
  virtual QString name();
  virtual void undo();
  virtual void redo();

private:
  ContactEntryList *mDocument;
  QString entryKey;
  ContactEntry *ce;

};

// all commands need to be changed so that instead of referencing
// a pabListViewItem they reference an entry key
class PwPasteCommand : public Command
{
public:
  PwPasteCommand(ContactEntryList *doc, QString clipboard );
  virtual QString name();
  virtual void undo();
  virtual void redo();

private:
    ContactEntryList *mDocument;
    QStringList keyList;
    QString clipboard;
};

class PwCutCommand : public Command
{
public:
  PwCutCommand(ContactEntryList *doc, const QStringList entryKeyList);
  virtual QString name();
  virtual void undo();
  virtual void redo();

private:
  ContactEntryList *mDocument;
  QStringList keyList;
  QString clipText;
  QString oldText;
};

class PwNewCommand : public Command
{
public:
  PwNewCommand(ContactEntryList *doc, ContactEntry *ce );
  ~PwNewCommand();
  virtual QString name();
  virtual void undo();
  virtual void redo();

private:
  ContactEntryList *mDocument;
  QString entryKey;
  ContactEntry *mCe;
};

class PwEditCommand : public Command
{
public:
  PwEditCommand(ContactEntryList *doc,
                QString entryKey,
                ContactEntry *oldCe, 
                ContactEntry *newCe );
  virtual ~PwEditCommand();
  virtual QString name();
  virtual void undo();
  virtual void redo();

private:
  ContactEntryList *mDocument;
  QString entryKey;
  ContactEntry *oldCe;
  ContactEntry *newCe;
};

#endif
